################################################################################
# Licensed Materials - Property of IBM Copyright IBM Corporation 2012. All Rights Reserved.
# U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP
# Schedule Contract with IBM Corp.
################################################################################

require "devops_library/providers/library_system_provider"

################################################################################
# Class Definition                                                             #
################################################################################
module DevopsLibrary
  module Providers
    class RubygemsSystemProvider < LibrarySystemProvider
      def initialize(args)
  
        if args[:gem_host].nil?
          raise ArgumentError, "A value for :gem_host is required."
        end
  
        @gem_host = args[:gem_host]
      end

      def create_package(root_folder, metadata={})
        if validate_root_folder(root_folder)
          gemspec_file = get_gemspec_filename(root_folder)
  
          Dir.chdir(root_folder) do
            `gem build #{gemspec_file}`
            if 0 != $?.to_i
              raise "gem build #{gemspec_file} failed"
            end
          end
        end
      end

      def publish_package(package)
        `gem inabox #{package} -qg #{@gem_host}`
        # if 0 != $?.to_i
        #           raise "gem inabox #{package} -qg #{@gem_host} failed"
        #         end
      end

      private
      ##############################################################################
      # Private Methods                                                            #
      ##############################################################################  
    
      def validate_root_folder(root_folder)
      
        unless File.directory?(root_folder)
          return false
        end
  
        if Dir["#{root_folder}/*.gemspec"].length == 1
          return true
        end
  
        return false
  
      end

      def get_gemspec_filename(root_folder)
        return File.expand_path(Dir["#{root_folder}/*.gemspec"][0])
      end

    end
  end
end
