require 'devops_core/crypto_util'

module Weaver
  module Core
    # Holds encrypted text such as passwords.  This value type only holds the ciphertext.
    # To get the plaintext call CryptoUtil
    # 
    # @see Weaver::Dsl::AbstractStatement#decrypt
    #
    # Author:: Ed Snible
    class EncryptedValue

      # The cipher text
      attr_reader :ciphertext
            
      # * +cipher+ Base64 encoded ciphertext
      def initialize(cipher)
        @ciphertext = cipher
      end

      # Generate the string "decrypt('<CIPHERTEXT')"
      def to_s
        "decrypt(#{ciphertext.inspect()})"
      end
      
      # Generate the string "decrypt('<CIPHERTEXT')"
      def inspect
        "decrypt(#{ciphertext.inspect})"
      end
      
      # Create a new EncryptedValue representing +cipher+
      # * +cipher+ Base64 encoded ciphertext
      def EncryptedValue.from_ciphertext(cipher)
        return EncryptedValue.new(cipher)
      end
      
      # Create a new EncryptedValue representing +plaintext+
      # * +plaintext+ Base64 encoded ciphertext
      # * +keyfile+ optional ezcrypto keyfile
      def EncryptedValue.from_plaintext(plaintext, keyfile = nil)
        encoded = DevopsCore::CryptoUtil.encrypt(plaintext, keyfile)
        base64 = Base64.encode64(encoded).gsub("\n", '')
        return EncryptedValue.new(base64)
      end
    end
  end
end