require 'weaver/core/base'
require 'weaver/core/attribute'
require 'weaver/core/generation'

module Weaver
  module Core
    # Represents a (possibly virtual) network card
    # @see Weaver::Dsl::NicStatement
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    class NetworkInterfaceController < Base     
      include Weaver::Core::Generation 
      
      # The firewall ports that may pass
      attr_accessor :firewall
      
      # Known hostnames for this NIC
      attr_reader :hostnames
      
      # * +id+ A symbol for the NetworkInterfaceController, e.g. +:nic0+
      # * +parent+ The enclosing Base
      def initialize(id, parent)
        super(id, parent)
        @type = :nic
        
        @firewall = nil
        @hostnames = []
        
        amd[:ip_address] = Attribute.new(:ip_address, self, true)
        amd[:ip_address].implicit = true
        amd[:ip_address].required = false
        amd[:ip_address].add_validator(:valid_ip)
      end
      
      # Make sure NetworkInterfaceController#clone creates a deep copy not a shallow copy
      # * +other+ Another NetworkInterfaceController
      def initialize_copy(other)
        super

        @firewall = other.firewall.clone unless other.firewall.nil?
        @hostnames = other.hostnames.clone
      end
      
      # The hostname this NIC's IP address is known by.  If this NIC has multiple DNS names, any of the names may be returned.
      def hostname
        return hostnames.length > 0 ? hostnames[0] : nil
      end
      
    end
  end
end