require 'weaver/core/base.rb'

module Weaver
  module Core
    
    # A mixin for classes such as Node which may contain Policy
    # 
    # Author:: Thomas Gschwind <thg@zurich.ibm.com>
    # Copyright:: IBM
    module PolicyContainerMixin
      # Let all classes that include PolicyContainerMixin have a method to return their policies
      def self.included(baseclazz)
        baseclazz.class_exec {
          def policies
            # Since mixins have no initialize() to set up @policies, initialize @policies lazily
            @policies ||= {} unless frozen?
            return @policies || {}
          end
        }
        
        baseclazz.register_initialize_copy_handler(:initialize_copy_policies)
        #baseclazz.register_validate_handler(:validate_policies)
        baseclazz.register_child_enumerator(:policies)           
      end
      
      def initialize_copy_policies(other_policy_container)
        # At this point, self.initialize_copy should have cloned @policies but the values are shallow.
        @policies = {}
        other_policy_container.policies.each do | id, policy |
          dup = policy.clone()
          dup.scope = self
          @policies[id] = dup
        end
      end
      
      def validate_policies(out_messages, desired_phases = nil)
        policies.each { |id, policy| policy.validate_collect(out_messages, desired_phases) }
      end
      
    end
    
  end
end