
module Weaver
  module Core
    # Represents a validator
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    class Validator

      # Ruby Proc which implements the validator
      attr_reader :proc
      
      # Symbol representing Weaver type such as :attribute or :any which controls what kinds of things the validator may be attached to
      attr_reader :type
      
      # Validation phase such as :predeployment
      attr_accessor :phase
      
      # * +id+ Symbol naming the validator
      # * +type+ Symbol representing Weaver type such as :attribute or :any which controls what kinds of things the validator may be attached to
      # * +proc+ Ruby Proc which implements the validator
      def initialize(id, type, proc)
        raise WeaverError.new("id must be a Symbol") unless id.kind_of?(Symbol)
        raise WeaverError.new("type must be a Symbol") unless type.kind_of?(Symbol)
        raise WeaverError.new("proc must be a Proc") unless proc.kind_of?(Proc)
        
        @id = id
        @type = type
        @proc = proc
        @phase = :any
      end
      
    end
    
  end
end