require "weaver/core/logging"   
require "weaver/deployers/deployer"

module Weaver
  module Deployers
    module Plugins
      # The MockDeployer class is mainly for testing and does nothing.
      #
      class MockDeployer < Deployer

        #
        # Return and ID for a cloud type that this deployer supports
        #
        def self.id
          return :mock_cloud
        end 
        
        # Noop
        #             
        # * *Args*    :
        #   -+topology+ the environment topology that needs to  
        def self.serialize_topology_preprocessor(topology)           
          topology
        end

        # Noop
        #
        # * *Args*    :
        #   -+properties+ A list of add properties configuring the deployer.
        def initialize(properties = {}, &reporter)
          super          
        end

        # Noop. Adds fake hostname and IPs to each node.
        #
        # * *Args*    :
        #   -+environment+ An instance of an Topology object that represents the binding
        #                              of an application to an environment.
        #   -+deploy_parameters+ ignored.
        # * *Returns* :
        #   -+Environment+ The updated environment instance with runtime information such as hostname and ip.
        #
        def deploy(environment, deploy_parameters = {})
          infrastructures = environment.get_cloud_infrastructures()
          raise WeaverError.new("Multiple clouds in environment: #{infrastructures.map {|inf| inf.id }.inspect}") if infrastructures.size > 1
          raise WeaverError.new("No clouds in environment") if infrastructures.empty?
          infrastructure = infrastructures[0]
       
          # set dummy hostname
          infrastructure.unique_nodes.each do |k, v|
            v.hostname = "my-mock-vm.ibm.com"
            v.ip_address = "1.2.3.4"
          end
          return environment
        end

      end
    end
  end
end 