require 'weaver/core/policy'
require 'weaver/dsl/depends_on_statement'

module Weaver
  module Dsl
    
    # The AutomationStatement implements the processing of the language element 'automation'.
    #
    # Author:: Florian Rosenberg   
    # Author:: Ed Snible  
    # @see Automation 
    class AutomationStatement < AbstractStatement
      include DependsOnStatement

      # * +id+ Symbol representing the entity, or a Core::Automation which is the entity
      # * +parent_statement+ The enclosing AbstractStatement
      def initialize(id_or_model, parent_statement)
        @@logger.debug "automation[id: #{id_or_model.kind_of?(Base) ? id_or_model.id : id_or_model.inspect }]"
        super(id_or_model, Automation, parent_statement)
      end

      # TODO remove
      def type(type)
        @@logger.warn "automation { type :#{type} } syntax deprecated"
        model.type = type
      end
            
      # The language +script_type+ statement.
      #
      # See {+script_type+}[link:STATEMENTS_rdoc.html#label-script_type]
      # * *Args*    :
      #   - +type+ -> Symbol, for example <code>:sh</code>
      def script_type(type)
        model.script_type = type
      end
            
      # The language +parameter+ statement.
      # * *Args* :
      #   - +parameter+ -> String | <Hash of name -> value>
      def parameter(parameter)
        @@logger.debug "parameter[#{parameter}]"
        if parameter.kind_of?(String)
          attr = Attribute.new(parameter.symbolize, model)
          attr.name = name
          attr.weaver_origin = weaver_stackframe
          model.amd[attr.id] = attr 
        elsif parameter.kind_of?(Hash) && parameter.length == 1
          name = parameter.keys.to_a[0]
          attr = Attribute.new(name.symbolize, model)
          attr.name = name
          attr.value = parameter.values.to_a[0]
          attr.weaver_origin = weaver_stackframe
          model.amd[attr.id] = attr 
        else
          model.add_validation_failure("Parameter invalid name => value pair: ':#{parameter}'.")
        end
      end
      
      # The language +parameters+ statement.
      # * *Args* :
      #   - +parameters+ -> Hash of names -> values
      def parameters(parameters)
        @@logger.debug "parameters[#{parameters}]"
        parameters.each do |name,val|
          id = name.symbolize
          attr = Attribute.new(id, model)
          attr.name = name
          attr.value = val
          attr.weaver_origin = weaver_stackframe
          model.amd[id] = attr 
        end 
      end
      
    end
  end

end
