require 'weaver/core/weaver_error'
require 'weaver/core/import'
require 'weaver/dsl/abstract_statement'
require 'weaver/dsl/component_container'
require 'weaver/dsl/node_container'
require 'weaver/dsl/topology_container_statement'
require 'weaver/dsl/import_container_statement'

module Weaver
  module Dsl
    # The ImportStatement implements the processing of the language elements that may be
    # imported.
    #
    # Author:: Ed Snible
    # Author:: Florian Rosenberg
    # @see Import
    class ImportStatement < AbstractStatement
      include TopologyContainerStatement
      include ServiceContainerStatement
      include ComponentContainer
      include NodeContainer
      include AutomationContainer
      include LoadDriverContainer
      include PolicyContainer
      include ImportContainerStatement

      # * +id+ Symbol representing the entity, or a Core::Import which is the entity
      # * +parent_statement+ The enclosing AbstractStatement
      def initialize(id, parent_statement)
        super(id, Import, parent_statement)
      end

      # Process keyword 'type'
      def type(clazz, &block)
        @@logger.debug "(within import) type[#{clazz.inspect}]"
        root_statement.type(clazz, &block)
      end
      
    end

  end

end