require 'weaver/dsl/firewall_statement'

module Weaver
  module Dsl
    # The NicStatement implements the processing of the language element 'nic'.
    # {Weaver::Core::NetworkInterfaceController}s support statements: include, firewall, hostname
    #
    # Author:: Ed Snible
    class NicStatement < AbstractStatement

      # * +id+ A symbol for the NetworkInterfaceController, e.g. +:nic0+
      # * +parent_statement+ AbstractStatement
      def initialize(id, parent_statement)
        @@logger.debug "nic[id: #{id} on #{parent_statement.model.type} :#{parent_statement.model.id}]"
        super(id, :nic, parent_statement)
      end

      # Parse firewall { } using a FirewallStatement
      # *block* A block containing Weaver statements about the NIC
      def firewall(&block)
        return model.add_validation_failure("Firewall redefinition") unless @model.firewall.nil?
                
        firewall_statement = FirewallStatement.new(self)
        @model.firewall = firewall_statement.model
        firewall_statement.instance_eval &block unless block.nil?
      end
      
      # NetworkInterfaceControllers can have multiple hostnames; this method adds a hostname
      # *hostname* a DNS hostname
      def hostname(hostname)
        return model.add_validation_failure("Hostnames must be strings") unless hostname.kind_of?(String)
        
        model.hostnames << hostname
      end

      # Summarize
      def to_s
        "nic[id: #{@id}, name: #{@name}]"
      end
      
    end

  end

end
