module Weaver
  module Dsl
    # Mix in UserContainer in statements that may contain {UserStatement}
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    module UserContainer
      include Weaver::Core
      
      # * *Args* :
      #   -+base+ the AbstractStatement being mixed with
      def self.included(base)
        base.register_statement_handler([:user_handler])
      end

      # * *Args* :
      #   -+sym+ <Symbol representing Weaver type, e.g. <code>:user</code>
      #   -+id+ Symbol naming entity
      #   -+block+ optional block of additional metadata
      def user_handler(sym, id, &block)
        return nil unless is_a?(Weaver::Dsl::UserContainer) && root_statement.model.weaver_extended_type?(sym, User)
        return nil unless id.kind_of?(Symbol) 
        
        user(id, root_statement.model.prototype_instance(sym, id, model, weaver_stackframe), &block)
      end

      # If prototype is nil, we will create a core Group, otherwise inherit attributes,
      # default attribute values, and validators from the prototype
      def user(id, prototype = nil, &block)
        @@logger.debug "user[(#{prototype.nil? ? 'user' : prototype.type}): #{id}]"
        
        return if fail_id_redefinition(id, User)
        user_model = @model.users[id]
        if user_model.nil?
          user_model = prototype || root_statement.model.prototype_instance(:user, id, model, weaver_stackframe)
          @model.users[id] = user_model
        else
          root_statement.model.widen(user_model, prototype) unless prototype.nil?
        end
        
        unless block.nil?
          user = UserStatement.new(user_model, self)
          user.instance_eval &block
        end
         
        return user.model
      end
    end
  end
end