require 'erubis'

module Weaver
  module Generators
    # This class is used to generate a continuous validation rule from deployment runtime information learned 
    #during the Weaver analytics phase
    class ContValRuleGenerator
      include Weaver::Core
      
      @@ARTIFACT_LIBRARY_DIR = '/opt/weaver/rules/state_checking_library'
      @@STATE_TYPES = {
        :process => "#{@@ARTIFACT_LIBRARY_DIR}/check_process.sh",
        :port => "#{@@ARTIFACT_LIBRARY_DIR}/check_port.rb"
      }
      
      # Generates a continuous validation rule
      # * *Args*    :
      #   - +rule_id+ -> the id of the rule
      #   - +node_id_regex+ -> regex corresponding to the target nodes of this rule
      #   - +command_line+ -> command that will be executed by this rule
      #   - +type+ -> Type corresponding to the rule runtime, e.g., :cron
      #   - +period+ -> Time interval (in minutes) between consecutive runs of the rule
      #   - +artifact_id+ -> id of the artifact required by this rule
      #
      # * *Returns* :
      #   - A String containing the Weaver code of the rule
      #
      def self.generate_contval_rule(rule_id, node_id_regex, command_line, type, period, artifact_id = nil)
        eruby = Erubis::Eruby.new(File.read(File.join(File.dirname(__FILE__), '..', '..', '..', 'data/templates/weaver_core_contval_rule.erb')))
        eruby.result({:rule_id=>rule_id, :node_id_regex=>node_id_regex, :command_line=>command_line, :type=>type, :period=>period, :artifact_id=>artifact_id}) 
      end
      
      def self.state_types
        @@STATE_TYPES
      end
    end
  end
end